@@ -1,174 +1,46 @@
 # Mail::Internet.pm
 #
-# Copyright (c) 1995 Graham Barr <gbarr@ti.com>. All rights
+# Copyright (c) 1995-8 Graham Barr <gbarr@pobox.com>. All rights
 # reserved. This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
+#
 
 package Mail::Internet;
 use strict;
 
-=head1 NAME
-
-Mail::Internet - manipulate Internet format (RFC 822) mail messages
-
-=head1 SYNOPSIS
-
-    use Mail::Internet;
-
-=head1 DESCRIPTION
-
-This package provides a class object which can be used for reading, creating,
-manipulating and writing a message with RFC822 compliant headers.
-
-=head1 CONSTRUCTOR
-
-=over 4
-
-=item new ( [ ARG ], [ OPTIONS ] )
-
-C<ARG> is optiona and may be either a file descriptor (reference to a GLOB)
-or a reference to an array. If given the new object will be
-initialized with headers either from the array of read from 
-the file descriptor.
-
-C<OPTIONS> is a list of options given in the form of key-value
-pairs, just like a hash table. Valid options are
-
-=over 8
-
-=item B<Header>
-
-The value of this option should be a C<Mail::Header> object. If given then
-C<Mail::Internet> will not attempt to read a mail header from C<ARG>, if
-it was specified.
-
-=item B<Body>
-
-The value of this option should be a reference to an array which contains
-the lines for the body of the message. If given then
-C<Mail::Internet> will not attempt to read the body from C<ARG>, if
-it was specified.
-
-=back
-
-The Mail::Header options C<Modify>, C<MailFrom> and C<FoldLength> may
-also be given.
-
-=back
-
-=head1 METHODS
-
-=over 4
-
-=item body ()
-
-Returns the body of the message. This is a reference to an array.
-Each entry in the array represents a single line in the message.
-
-=item print_header ( [ FILEHANDLE ] )
-
-=item print_body ( [ FILEHANDLE ] )
-
-=item print ( [ FILEHANDLE ] )
-
-Print the header, body or whole message to file descriptor I<FILEHANDLE>.
-I<$fd> should be a reference to a GLOB. If I<FILEHANDLE> is not given the
-output will be sent to STDOUT.
-
-    $mail->print( \*STDOUT );  # Print message to STDOUT
-
-=item head ()
-
-Returns the C<Mail::Header> object which holds the headers for the current
-message
-
-=back
-
-=head1 UTILITY METHODS
-
-The following methods are more a utility type than a manipulation
-type of method.
-
-=over 4
-
-=item remove_sig ( [ NLINES ] )
-
-Attempts to remove a users signature from the body of a message. It does this 
-by looking for a line equal to C<'-- '> within the last C<NLINES> of the
-message. If found then that line and all lines after it will be removed. If
-C<NLINES> is not given a default value of 10 will be used. This would be of
-most use in auto-reply scripts.
-
-=item tidy_body ()
-
-Removes all leading and trailing lines from the body that only contain
-white spaces.
-
-=item reply ()
-
-Create a new object with header initialised for a reply to the current 
-object. And the body will be a copy of the current message indented.
-
-=item add_signature ( [ FILE ] )
-
-Append a signature to the message. C<FILE> is a file which contains
-the signature, if not given then the file "$ENV{HOME}/.signature"
-will be checked for.
-
-=item smtpsend ()
-
-Send a Mail::Internet message via SMTP
-
-The message will be sent to all addresses on the To, Cc and Bcc
-lines. The SMTP host is found by attempting connections first
-to hosts specified in C<$ENV{SMTPHOSTS}>, a colon separated list,
-then C<mailhost> and C<localhost>.
-
-=item nntppost ()
-
-Post an article via NNTP, require News::NNTPClient.
-
-=item escape_from ()
-
-It can cause problems with some applications if a message contains a line
-starting with C<`From '>, in particular when attempting to split a folder.
-This method inserts a leading C<`>'> on anyline starting with C<`From '>
-
-=item unescape_from ()
-
-This method will remove the escaping added bu escape_from
-
-=back
-
-=head1 SEE ALSO
-
-L<Mail::Header>
-L<Mail::Address>
-
-=head1 AUTHOR
-
-Graham Barr <gbarr@ti.com>
-
-=head1 COPYRIGHT
-
-Copyright (c) 1995 Graham Barr. All rights reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms
-as Perl itself.
-
-=cut
-
 require 5.002;
 
 use Carp;
 use AutoLoader;
 use Mail::Header;
-use vars qw($VERSION @ISA);
+use vars qw($VERSION);
 
-$VERSION = do { my @r=(q$Revision: 1.28 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r};
-@ISA     = qw(AutoLoader);
+BEGIN {
+    $VERSION = "1.32";
+    *AUTOLOAD = \&AutoLoader::AUTOLOAD;
+
+    unless(defined &UNIVERSAL::isa) {
+	*UNIVERSAL::isa = sub {
+	    my($obj,$type) = @_;
+	    my $pkg = ref($obj) || $obj;
+	    my @pkg = ($pkg);
+	    my %done;
+	    while(@pkg) {
+        	$pkg = shift @pkg;
+		return 1 if $pkg eq $type;
+        	next if exists $done{$pkg};
+        	$done{$pkg} = 1;
+
+		no strict 'refs';
+
+        	unshift @pkg,@{$pkg . "::ISA"}
+        	    if(@{$pkg . "::ISA"});
+	    }
+	    undef;
+	}
+    }
+}
 
-# stop import being inherited from AutoLoader by use Mail::Internet :-(
-sub import {}
 
 sub new
 {
@@ -383,6 +255,23 @@ sub print
     and $me->print_body($fd);
 }
 
+sub as_string
+{
+ my $me = shift;
+
+ $me->head->as_string . "\n" . join '', @{ $me->body };
+}
+
+sub as_mbox_string
+{
+ my $me = shift->dup;
+ my $escaped = shift;
+
+ $me->head->delete('Content-Length');
+ $me->escape_from unless $escaped;
+ $me->as_string . "\n";
+}
+
 sub remove_sig
 {
  my $me = shift;
@@ -439,6 +328,7 @@ use Mail::Address;
  my $pkg = ref $me;
  my @reply = ();
 
+ local *MAILHDR;
  if(open(MAILHDR,"$ENV{HOME}/.mailhdr")) 
   {
    # User has defined a mail header template
@@ -610,6 +500,7 @@ sub sign
 
    if(open(SIG,$sig))
     {
+     local $_;
      while(<SIG>) { last unless /\A(--)?\s*\Z/; }
 
      @sig = ($_,<SIG>,"\n");
@@ -630,6 +521,37 @@ sub sign
   }
 }
 
+sub _prephdr {
+    my $hdr = shift;
+
+    $hdr->delete('From '); # Just in case :-)
+
+    # An original message should not have any Received lines
+
+    $hdr->delete('Received');
+
+    $hdr->replace('X-Mailer', "Perl5 Mail::Internet v" . $Mail::Internet::VERSION);
+
+    my $name = eval { (getpwuid($>))[6] } || $ENV{NAME} || "";
+
+    while($name =~ s/\([^\(\)]*\)//) { 1; }
+
+    if($name =~ /[^\w\s]/) {
+	$name =~ s/"/\"/g;
+	$name = '"' . $name . '"';
+    }
+
+    my $from = sprintf "%s <%s>", $name, mailaddress();
+    $from =~ s/\s{2,}/ /g;
+
+    my $tag;
+
+    foreach $tag (qw(From Sender)) {
+	$hdr->add($tag,$from)
+	    unless($hdr->get($tag));
+    }
+}
+
 sub smtpsend;
 
 use Carp;
@@ -637,109 +559,120 @@ use Mail::Util qw(mailaddress);
 use Mail::Address;
 use Net::Domain qw(hostname);
 use Net::SMTP;
-
+use strict;
 
  sub smtpsend 
 {
- my $src  = shift;
- my($mail,$smtp,@hosts);
+    my $src  = shift;
+    my %opt = @_;
+    my $host = $opt{Host};
+    my $noquit = 0;
+    my $smtp;
 
- require Net::SMTP;
+    unless(defined($host)) {
+	my @hosts = qw(mailhost localhost);
+	unshift(@hosts, split(/:/, $ENV{SMTPHOSTS})) if(defined $ENV{SMTPHOSTS});
 
- @hosts = qw(mailhost localhost);
- unshift(@hosts, split(/:/, $ENV{SMTPHOSTS})) if(defined $ENV{SMTPHOSTS});
+	foreach $host (@hosts) {
+	    $smtp = eval { Net::SMTP->new($host) };
+	    last if(defined $smtp);
+	}
+    }
+    elsif(ref($host) && UNIVERSAL::isa($host,'Net::SMTP')) {
+	$smtp = $host;
+	$noquit = 1;
+    }
+    else {
+	$smtp = eval { Net::SMTP->new($host) };
+    }
 
- foreach $host (@hosts) {
-  $smtp = eval { Net::SMTP->new($host) };
-  last if(defined $smtp);
- }
+    return ()
+	unless(defined $smtp);
 
- croak "Cannot initiate a SMTP connection" unless(defined $smtp);
+    $smtp->hello( hostname() );
+    my $hdr = $src->head->dup;
 
- $smtp->hello( hostname() );
- $mail = $src->dup;
+    _prephdr($hdr);
 
- $mail->delete('From '); # Just in case :-)
+    # Who is it to
 
- $mail->replace('X-Mailer', "Perl5 Mail::Internet v" . $Mail::Internet::VERSION);
+    my @rcpt = map { ref($_) ? @$_ : $_ } grep { defined } @opt{'To','Cc','Bcc'};
+    @rcpt = map { $hdr->get($_) } qw(To Cc Bcc)
+	unless @rcpt;
+    my @addr = map($_->address, Mail::Address->parse(@rcpt));
 
- # Ensure the mail has the following headers
- # Sender, From, Reply-To
+    return ()
+	unless(@addr);
 
- my($from,$name,$tag);
+    $hdr->delete('Bcc'); # Remove blind Cc's
 
- $name = (getpwuid($>))[6] || $ENV{NAME} || "";
- while($name =~ s/\([^\(]*\)//) { 1; }
+    # Send it
 
- $from = sprintf "%s <%s>", $name, mailaddress();
- $from =~ s/\s{2,}/ /g;
+    my $ok = $smtp->mail( mailaddress() ) &&
+		$smtp->to(@addr) &&
+		$smtp->data(join("", @{$hdr->header},"\n",@{$src->body}));
 
- foreach $tag (qw(Sender From Reply-To))
-  {
-   $mail->add($tag,$from) unless($mail->get($tag));
-  }
+    $smtp->quit
+	unless $noquit;
 
- # An original message should not have any Received lines
-
- $mail->delete('Received');
-
- # Who is it to
-
- my @rcpt = ($mail->get('To', 'Cc', 'Bcc'));
- my @addr = map($_->address, Mail::Address->parse(@rcpt));
-
- return () unless(@addr);
-
- $mail->delete('Bcc'); # Remove blind Cc's
-
- # Send it
-
- my $ok = $smtp->mail( mailaddress() ) &&
-            $smtp->to(@addr) &&
-            $smtp->data(join("", @{$mail->header},"\n",@{$mail->body}));
-
- $smtp->quit;
-
- $ok ? @addr : ();
+    $ok ? @addr : ();
 }
 
 sub nntppost;
 
 use Mail::Util qw(mailaddress);
-
-
-require News::NNTPClient;
+use Net::NNTP;
+use strict;
 
  sub nntppost
 {
- my $mail = shift;
+    my $mail = shift;
+    my %opt = @_;
 
- my $groups = $mail->get('Newsgroups') || "";
- my @groups = split(/[\s,]+/,$groups);
+    my $groups = $mail->get('Newsgroups') || "";
+    my @groups = split(/[\s,]+/,$groups);
 
- return () unless @groups;
+    return ()
+	unless @groups;
 
- my $art = $mail->dup;
+    my $hdr = $mail->head->dup;
 
- $art->replace('X-Mailer', "Perl5 Mail::Internet v" . $Mail::Internet::VERSION);
+    _prephdr($hdr);
 
- unless($art->get('From'))
-  {
-   my $name = $ENV{NAME} || (getpwuid($>))[6];
-   while( $name =~ s/\([^\(]*\)// ) {1};
-   $art->replace('From',$name . " <" . mailaddress() . ">");
-  }
+    # Remove these incase the NNTP host decides to mail as well as me
+    $hdr->delete(qw(To Cc Bcc)); 
 
- # Remove these incase the NNTP host decides to mail as well as me
- $art->delete(qw(To Cc Bcc)); 
+    my $news;
+    my $noquit = 0;
+    my $host = $opt{Host};
 
- my $news = new News::NNTPClient;
- $news->post(@{$art->header},"\n",@{$art->body});
+    if(ref($host) && UNIVERSAL::isa($host,'Net::NNTP')) {
+	$news = $host;
+	$noquit = 1;
+    }
+    else {
+	my @opt = ();
 
- my $code = $news->code;
- $news->quit;
+	push(@opt, $opt{'Host'});
 
- return 240 == $code ? @groups : ();
+	push(@opt, 'Port', $opt{'Port'})
+	    if exists $opt{'Port'};
+
+	push(@opt, 'Debug', $opt{'Debug'})
+	    if exists $opt{'Debug'};
+
+	$news = new Net::NNTP(@opt)
+	    or return ();
+    }
+
+    $news->post(@{$hdr->header},"\n",@{$mail->body});
+
+    my $code = $news->code;
+
+    $news->quit
+	unless $noquit;
+
+    return 240 == $code ? @groups : ();
 }
 
 sub escape_from
@@ -766,3 +699,205 @@ sub unescape_from
 
 
 
+=head1 NAME
+
+Mail::Internet - manipulate Internet format (RFC 822) mail messages
+
+=head1 SYNOPSIS
+
+    use Mail::Internet;
+
+=head1 DESCRIPTION
+
+This package provides a class object which can be used for reading, creating,
+manipulating and writing a message with RFC822 compliant headers.
+
+=head1 CONSTRUCTOR
+
+=over 4
+
+=item new ( [ ARG ], [ OPTIONS ] )
+
+C<ARG> is optiona and may be either a file descriptor (reference to a GLOB)
+or a reference to an array. If given the new object will be
+initialized with headers and body either from the array of read from 
+the file descriptor.
+
+C<OPTIONS> is a list of options given in the form of key-value
+pairs, just like a hash table. Valid options are
+
+=over 8
+
+=item B<Header>
+
+The value of this option should be a C<Mail::Header> object. If given then
+C<Mail::Internet> will not attempt to read a mail header from C<ARG>, if
+it was specified.
+
+=item B<Body>
+
+The value of this option should be a reference to an array which contains
+the lines for the body of the message. Each line should be terminated with
+C<\n> (LF). If Body is given then C<Mail::Internet> will not attempt to
+read the body from C<ARG> (even if it is specified).
+
+=back
+
+The Mail::Header options C<Modify>, C<MailFrom> and C<FoldLength> may
+also be given.
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item body ()
+
+Returns the body of the message. This is a reference to an array.
+Each entry in the array represents a single line in the message.
+
+=item print_header ( [ FILEHANDLE ] )
+
+=item print_body ( [ FILEHANDLE ] )
+
+=item print ( [ FILEHANDLE ] )
+
+Print the header, body or whole message to file descriptor I<FILEHANDLE>.
+I<$fd> should be a reference to a GLOB. If I<FILEHANDLE> is not given the
+output will be sent to STDOUT.
+
+    $mail->print( \*STDOUT );  # Print message to STDOUT
+
+=item as_string ()
+
+Returns the message as a single string.
+
+=item as_mbox_string ( [ ALREADY_ESCAPED ] )
+
+Returns the message as a string in mbox format.  C<ALREADY_ESCAPED>, if
+given and true, indicates that ->escape_from has already been called on
+this object.
+
+=item head ()
+
+Returns the C<Mail::Header> object which holds the headers for the current
+message
+
+=back
+
+=head1 UTILITY METHODS
+
+The following methods are more a utility type than a manipulation
+type of method.
+
+=over 4
+
+=item remove_sig ( [ NLINES ] )
+
+Attempts to remove a users signature from the body of a message. It does this 
+by looking for a line equal to C<'-- '> within the last C<NLINES> of the
+message. If found then that line and all lines after it will be removed. If
+C<NLINES> is not given a default value of 10 will be used. This would be of
+most use in auto-reply scripts.
+
+=item tidy_body ()
+
+Removes all leading and trailing lines from the body that only contain
+white spaces.
+
+=item reply ()
+
+Create a new object with header initialised for a reply to the current 
+object. And the body will be a copy of the current message indented.
+
+=item add_signature ( [ FILE ] )
+
+Append a signature to the message. C<FILE> is a file which contains
+the signature, if not given then the file "$ENV{HOME}/.signature"
+will be checked for.
+
+=item smtpsend ( [ OPTIONS ] )
+
+Send a Mail::Internet message via SMTP, requires Net::SMTP
+
+The return value will be a list of email addresses that the message was sent
+to. If the message was not sent the list will be empty.
+
+Options are passed as key-value pairs. Current options are
+
+=over 4
+
+=item Host
+
+Name of the SMTP server to connect to, or a Net::SMTP object to use
+
+If C<Host> is not given then the SMTP host is found by attempting
+connections first to hosts specified in C<$ENV{SMTPHOSTS}>, a colon
+separated list, then C<mailhost> and C<localhost>.
+
+=item To
+
+=item Cc
+
+=item Bcc
+
+Send the email to the given addresses, each can be either a string or
+a reference to a list of email addresses. If none of C<To>, <Cc> or C<Bcc>
+are given then the addresses are extracted from the message being sent.
+
+=back
+
+=item nntppost ( [ OPTIONS ] )
+
+Post an article via NNTP, requires Net::NNTP.
+
+Options are passed as key-value pairs. Current options are
+
+=over 4
+
+=item Host
+
+Name of NNTP server to connect to, or a Net::NNTP object to use.
+
+=item Port
+
+Port number to connect to on remote host
+
+=item Debug
+
+Debug value to pass to Net::NNTP, see <Net::NNTP>
+
+=back
+
+=item escape_from ()
+
+It can cause problems with some applications if a message contains a line
+starting with C<`From '>, in particular when attempting to split a folder.
+This method inserts a leading C<`>'> on anyline that matches the regular
+expression C</^>*From/>
+
+=item unescape_from ()
+
+This method will remove the escaping added by escape_from
+
+=back
+
+=head1 SEE ALSO
+
+L<Mail::Header>
+L<Mail::Address>
+
+=head1 AUTHOR
+
+Graham Barr <gbarr@pobox.com>
+
+=head1 COPYRIGHT
+
+Copyright (c) 1995-7 Graham Barr. All rights reserved. This program is free
+software; you can redistribute it and/or modify it under the same terms
+as Perl itself.
+
+=cut
+
+
